#ifndef _SSLBF2A_H_

#define _SSLBF2A_H_

typedef unsigned char BYTE;

#include <openssl/evp.h>

// use a 128-bit key like they say instead of the max key length.
#define EVP_BLOWFISH_KEY_SIZE       16
#define KEY_LENGTH EVP_BLOWFISH_KEY_SIZE
// Header means: blowfish cbc, iv is the first 8 bytes, 
// salt is the next 8 bytes, and the password is sha1 hashed.
#define BF_HEADER "Salted__"

#define MAX_PACKET_SIZE 1024

#ifdef _DEBUG
#define VERBOSE 1
#else
#define VERBOSE 0
#endif

int seed_prng(int bytes);
void get_password(char *pass, int len, int encrypting);
int encrypt_packet(BYTE *data, int data_size, const char *password, int pass_len, BYTE *out, int *enc_size, int32_t packet_number);
int decrypt_packet(const BYTE *enc_data, int enc_size, const char *password, int pass_len, BYTE *out, int *data_size, int32_t *packet_number);
//BYTE *simple_digest(char *buf, unsigned int len, int *olen);
void generate_key(BYTE *key, const char *password, const BYTE *salt, BYTE *iv, int b);
void select_random_salt(char *salt, int b);

void print_hex(BYTE *data, int b);

#ifdef __SSLBF2B__

void generate_key(BYTE *key, const char *password, const BYTE *salt, BYTE *iv, int b);
void generate_bin_key(BYTE *key, const BYTE *password, int pass_len, const BYTE *salt, BYTE *iv, int b);
int encrypt_data(EVP_CIPHER_CTX *ctx, char *output, const char *data, int inl, int *rb);
int decrypt_data(EVP_CIPHER_CTX *ctx, char *output, const char *ct, int inl);
int HMAC_create(const BYTE *data, int dlen, const BYTE *key, int klen, BYTE *check, int clen);
int HMAC_check(const BYTE *data, int dlen, const BYTE *key, int klen, BYTE *check, int clen);

#endif

#endif
